require 'component'
require 'rational'
require 'test/unit'

class ComponentTester < Test::Unit::TestCase
  def validate(n, str)
    assert_equal n, parse(str)
    assert_equal str, string(n)
  end

  def parse(str)
    Component.LLDN.parse(str)
  end

  def string(n)
    Component.LLDN.string_expr(n.to_r)
  end

  def test_roundtrip_1
    validate 0, ''
    validate 1, ''
    validate 2, ''
    validate 3, ''
    validate 4, ''
    validate 5, ''
    validate 6, 'ϻ'
    validate 7, ''
    validate 8, 'Ȭ'
    validate 9, ''
  end

  def test_roundtrip_2
    validate 10, ''
    validate 11, ''
    validate 12, ''
    validate 20, ''
    validate 21, '󽽰'
    validate 57, '޽'
    validate 99, '彽'
    validate 100, 'ɴ'
    validate 101, 'ɴ'
    validate 999, 'ɴ彽'
    validate 1000, ''
    validate 1001, ''
    validate 9999, 'ɴ彽'
    validate 10000, ''
    validate 10001, ''
    validate 10101, 'ɴ'
  end

  def test_roundtrip_3
    validate 10_0000, ''
    validate 10_0001, ''
    validate 20_0000, ''
    validate 100_0000, 'ɴ'
    validate 200_0000, 'ɴ'
    validate 101_0101, 'ɴɴ'
    validate 100_1001, 'ɴ'
    # validate 1000_0000, ''  # ˤϡְפĤʤ
    validate 1_0000_0000, '첯'
    validate 1_0001_0001, '첯'
  end

  def test_roundtrip_4
    validate 10**12, ''
    validate 10**16, ''
    validate 10**20, 'Զ'
    validate 10**24, 'ͽ'
    validate 10**28, ''
    validate 10**32, ''
    validate 10**36, ''
    validate 10**40, ''
    validate 10**44, ''
    validate 10**48, ''
  end

  def test_roundtrip_5
    validate 10**56, '칱Ϻ'
    validate 10**64, '찤ε'
    validate 10**72, 'ͳ¿'
    validate 10**80, 'ԲĻ׵'
    validate 10**88, '̵'
  end

  def test_roundtrip_6
    validate 10**89, '̵'
    validate 10**90, 'ɴ̵'
    validate 10**91, '̵'
    validate 10**92, '̵'
    validate 10**93, '̵'
    validate 10**94, 'ɴ̵'
    validate 10**95, '̵'
    validate 10**95+1, '̵'
    validate 10**96-1, 'ɴ彽ɴ彽̵ɴ彽ɴ彽ԲĻ׵Ķɴ彽ɴ彽ͳ¿ɴ彽ɴ彽尤εɴ彽ɴ彽幱Ϻɴ彽ɴ彽˶ɴ彽ܶɴ彽ɴ彽¶ɴ彽¶ɴ彽ɴ彽ͽɴ彽Զɴ彽ɴ彽ɴ彽岯ɴ彽ɴ彽'
    assert_raise(::IntegerOverflow) {
      string(10**96)
    }
    assert_raise(::IntegerOverflow) {
      parse('첯̵')
    }
  end

  def test_sign
    assert_equal '', string(-4)
    assert_equal 'ɴ', string(-100)
    assert_equal '', string(-1000)
    assert_equal 'ɴ', string(-200_0005)
  end

  def test_yo
    assert_equal ';', string(Rational(9,2))
    assert_equal ';', string(Rational(-7,2))
  end
end
